module Main where

import Test.Hspec (hspec, it, shouldBe)
import Data.Function ((&))

depthIncrease :: (Num a, Ord a) => [a] -> Int
depthIncrease xs@(_:ys) = length $ filter (>0) $ zipWith (-) ys xs

slidingIncrease :: (Num a, Ord a) => Int -> [a] -> Int
slidingIncrease n l = depthIncrease $ map sum $ windows n l

windows :: Int -> [a] -> [[a]]
windows n l@(_:xs)
  | length l >= n = take n l : windows n xs
  | otherwise = []

testSample = [199, 200, 208, 210, 200, 207, 240, 269, 260, 263]

runTests = do
  hspec $ do
    it "calculates depth increase" $ do
      depthIncrease testSample `shouldBe` 7
    it "calculates sliding window increase" $ do
      slidingIncrease 3 testSample `shouldBe` 5

main = do
  runTests

  input <- getContents & fmap lines
  let depths = map read input

  putStr "--\nPart 1: "
  print $ depthIncrease depths
  putStr "Part 2: "
  print $ slidingIncrease 3 depths
