module Main where

import Test.Hspec (hspec, it, shouldBe)
import Data.Function ((&))

parse :: String -> (String, Int)
parse s = let (x:y:_) = words s in (x, read y)

follow :: [(String, Int)] -> (Int, Int)
follow = foldl go (0, 0)
  where
    go (pos, dep) (dir, x) = case dir of
      "forward" -> (pos+x, dep)
      "down"    -> (pos, dep+x)
      "up"      -> (pos, dep-x)
      _         -> (pos, dep)

follow' :: [(String, Int)] -> (Int, Int)
follow' dirs = (pos, dep)
  where
    (_, pos, dep) = foldl go (0, 0, 0) dirs
    go (aim, pos, dep) (dir, x) = case dir of
      "down"    -> (aim+x, pos, dep)
      "up"      -> (aim-x, pos, dep)
      "forward" -> (aim, pos+x, dep+(aim*x))
      _         -> (aim, pos, dep)

testSample = [("forward", 5), ("down", 5), ("forward", 8), ("up", 3), ("down", 8), ("forward", 2)]
runTests = do
  hspec $ do
    it "parses directions" $ do
      parse "forward 5" `shouldBe` ("forward", 5)
      parse "down 3" `shouldBe` ("down", 3)
    it "runs directions" $ do
      follow testSample `shouldBe` (15, 10)
    it "runs modified directions" $ do
      follow' testSample `shouldBe` (15, 60)

main = do
  runTests

  input <- getContents & fmap lines
  let directions = map parse input

  putStr "--\nPart 1: "
  print $ uncurry (*) $ follow directions
  putStr "Part 2: "
  print $ uncurry (*) $ follow' directions
