{-# LANGUAGE TupleSections #-}
module Main where

import Test.Hspec
import qualified Data.Map as M
import Data.Map(Map)
import Data.Void
import Text.Megaparsec
import Text.Megaparsec.Char

type Parser = Parsec Void String

type Population = Map Int Int

intListP :: Parser [Int]
intListP = integer `sepBy` char ','
  where
    integer :: Parser Int
    integer = read <$> some digitChar

------------------------------------------------------------------------------

mkPop :: [Int] -> Population
mkPop = M.fromListWith (+) . map (,1)

step :: Population -> Population
step pop = M.fromListWith (+) $ update $ M.assocs pop
  where
    update [] = []
    update ((0, v):xs) = (8, v):(6, v):update xs
    update ((k, v):xs) = (k-1, v):update xs

countFish :: Population -> Int
countFish = M.foldr (+) 0

simulate :: Int -> Population -> Population
simulate n pop = iterate step pop !! n

------------------------------------------------------------------------------

main = do
  runTests

  Right input <- parse intListP "(stdin)" <$> getContents
  let population = mkPop input

  putStr "--\nPart 1: "
  print $ countFish $ simulate 80 population
  putStr "Part 2: "
  print $ countFish $ simulate 256 population

------------------------------------------------------------------------------

runTests = hspec $ do
  it "parses input" $ do
    parse intListP "(test)" testInput `shouldBe` Right [3,4,3,1,2]
  it "initialises the population" $ do
    mkPop testSample `shouldBe` M.fromList [(1,1), (2,1), (3,2), (4,1)]
  it "updates the population" $ do
    step (mkPop [3,4,3,1,2]) `shouldBe` mkPop [2,3,2,0,1]
    step (mkPop [2,3,2,0,1]) `shouldBe` mkPop [1,2,1,6,0,8]
    step (mkPop [0,1,0,5,6,7,8]) `shouldBe` mkPop [6,0,6,4,5,6,7,8,8]
  it "counts fish" $ do
    countFish (mkPop testSample) `shouldBe` length testSample
    countFish (mkPop [6,0,6,4,5,6,7,8,8]) `shouldBe` length [6,0,6,4,5,6,7,8,8]
  it "simulates fish" $ do
    countFish (simulate 18 $ mkPop testSample) `shouldBe` 26
    countFish (simulate 80 $ mkPop testSample) `shouldBe` 5934
    countFish (simulate 256 $ mkPop testSample) `shouldBe` 26984457539

testInput = "3,4,3,1,2"
testSample = [3,4,3,1,2]
