module Main where

import Test.Hspec
import Data.Void
import Text.Megaparsec
import Text.Megaparsec.Char
import Text.Megaparsec.Char.Lexer

type Parser = Parsec Void String

listP :: Parser [Int]
listP = decimal `sepBy` char ','

fuel :: Int -> [Int] -> Int
fuel n xs = sum $ map (abs.(n-)) xs

fuel' :: Int -> [Int] -> Int
fuel' n xs = sum $ map cost xs
  where cost m = let k = abs (n-m) in k * (k+1) `div` 2

minFuel :: (Int -> [Int] -> Int) -> [Int] -> Int
minFuel usage xs = minimum $ map (`usage` xs) [minimum xs..maximum xs]

------------------------------------------------------------------------------

main = do
  runTests

  Right positions <- parse listP "(stdin)" <$> getContents

  putStr "--\nPart 1: "
  print $ minFuel fuel positions
  putStr "Part 2: "
  print $ minFuel fuel' positions

------------------------------------------------------------------------------

runTests = hspec $ do
  it "parses input" $ do
    parse listP "(test)" testInput `shouldBe` Right testSample
  it "calculates fuel usage" $ do
    fuel 2 testSample `shouldBe` 37
    fuel 1 testSample `shouldBe` 41
    fuel 3 testSample `shouldBe` 39
    fuel 10 testSample `shouldBe` 71
  it "calculates real fuel usage" $ do
    fuel' 2 testSample `shouldBe` 206
    fuel' 5 testSample `shouldBe` 168
  it "minimises fuel usage" $ do
    minFuel fuel testSample `shouldBe` 37
  it "minimises real fuel usage" $ do
    minFuel fuel' testSample `shouldBe` 168

testInput = "16,1,2,0,4,2,7,1,2,14"
testSample = [16,1,2,0,4,2,7,1,2,14]
